-------------------------------------------------------
--! @file bcd7seg_tb.vhd
--! @brief Testbench for BCD to 7-segment encoder
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2020-04-01
-------------------------------------------------------

entity bcd7seg_tb is
end entity bcd7seg_tb;

architecture testbench of bcd7seg_tb is

--  Declaration of the component that will be instantiated.  
  component bcd7seg
    port (
      EN : in  bit;
      D :  in  bit_vector (3 downto 0);
      S :  out bit_vector (6 downto 0)
      );
  end component bcd7seg;

  signal en : bit;
  signal d : bit_vector(3 downto 0);
  signal s : bit_vector(6 downto 0);
  
begin
  --  Component instantiation.
  DUT: entity work.bcd7seg (when_else) port map (
    EN => en,
    D => d,
    S => s
    );

  --  This process does the real job.
  stimulus_process: process is
    type pattern_type is record
      --  The inputs of the circuit.
      en : bit;
      d : bit_vector (3 downto 0);
      
      --  The expected outputs of the circuit.
      s : bit_vector (6 downto 0);
    end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (
        ('0',"1010","0000000"),
        ('1',"1010","0000000"),
        ('1',"0000","1111110"),
        ('1',"0001","0110000"),
        ('1',"0010","1101101"),
        ('1',"0011","1111001"),
        ('1',"0100","0110011"),
        ('1',"0101","1011011"),
        ('1',"0110","1011111"),
        ('1',"0111","1110000"),
        ('1',"1000","1111111"),
        ('1',"1001","1111011"));
        
  begin
    for k in patterns'range loop

      --  Set the inputs.
      en <= patterns(k).en;
      d <= patterns(k).d;
      
       --  Wait for the results.
      wait for 1 ns;
      
  --  Check the outputs.

      assert s = patterns(k).s        
        report "bad output" severity error;

    end loop;
    
    assert false report "end of test" severity note;
    
  --  Wait forever; this will finish the simulation.
    wait;
  end process;
  
end architecture testbench;
  

