-------------------------------------------------------
--! @file addsub44_tb.vhd
--! @brief Testbench for 4-bit adder/subtractor
--! @author Edson S. Gomi (gomi@usp.br)
--! @date 2018-10-07
-------------------------------------------------------

entity addsub4_tb is
end entity addsub4_tb;

architecture arch of addsub4_tb is

-- Component Declaration
component addsub4 is
port (
    op :  in bit; -- 0 is add, 1 is sub
    x :   in bit_vector (3 downto 0);
    y :   in bit_vector (3 downto 0);
    r :   out bit_vector (3 downto 0); -- result: x+y or x-y
    ovf : out bit -- Overflow
    );
end component addsub4;

-- Signals
signal op : bit;
signal x : bit_vector (3 downto 0);
signal y : bit_vector (3 downto 0);
signal r : bit_vector (3 downto 0);
signal ovf : bit;

begin
  -- Component Instantiation
  UADDSUB : component addsub4 port map (op,x,y,r,ovf);
    
  -- Stimulus process
  stimulus_process : process is
    type pattern_type is record
     -- The inputs of the circuit
     op : bit;
     x : bit_vector (3 downto 0);
     y : bit_vector (3 downto 0); 
     -- The expected outputs of the circuit
     r : bit_vector (3 downto 0);
     ovf : bit;
     end record;

    --  The patterns to apply.
    type pattern_array is array (natural range <>) of pattern_type;
    constant patterns : pattern_array :=
      (('0',"0001","0011","0100",'0'), -- +1+3 = +2
       ('0',"1111","0011","0010",'0'), -- -1+3 = +2
       ('0',"0101","0100","1001",'1'), -- +5+4 = +9
       ('1',"1011","0100","0111",'1'), -- -5-4 = -9 
       ('1',"1111","0011","1100",'0'), -- -1-3 = -4 
       ('1',"0001","0011","1110",'0'), -- +1-3 = -2
       ('1',"0111","0100","0011",'0'), -- +7-4 = +3
       ('0',"1001","0100","1101",'0')); -- -7+4 = -3

    begin

    wait for 1 ns;
      
    --  Check each pattern.
    for k in patterns'range loop
      --  Set the inputs.
      op <= patterns(k).op;
      x <= patterns(k).x;
      y <= patterns(k).y;
  
    --  Wait for the results.
    wait for 3 ns;
    --  Check the outputs.
    assert r = patterns(k).r
      report "bad check" severity error;
    assert ovf = patterns(k).ovf
      report "bad check" severity error;
    end loop;
    assert false report "end of test" severity note;
    --  Wait forever; this will finish the simulation.
      wait;
 end process;

end architecture arch;
